---
title: "Smoothed Passenger Miles"
format: html
---

We smoothing our data using a spline with 5 knots:

```{r}
#| label: fig_smoothed_5
#| echo: false
knitr::include_graphics("fig_smoothed_5.png")
```

And smooth it again using a spline with 10 knots

```{r}
#| label: fig_smoothed_10
#| echo: false
knitr::include_graphics("fig_smoothed_10.png")
```

